<ol class="toc">
{% for article in articles %}
<li class="toc-{{ article.level }}"><a href="#{{article.anchor}}">{{ article.enumerator }} {{ article.title }}</a></li>
{% if article.articles %}
{% include "wbwriter/templatetags/table_of_contents_articles.html" with articles=article.articles %}
{% endif %}
{% endfor %}
</ol>
