# Generated by Django 4.2.9 on 2024-02-20 10:19

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbwriter", "0009_dependantarticle"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="article",
            options={
                "permissions": [("administrate_article", "Can administrate Articles.")],
                "verbose_name": "Article",
                "verbose_name_plural": "Articles",
            },
        ),
    ]
